-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: PetDPS")

L:RegisterTranslations("enUS", function() return {
	["Pet Damage per Second"] = true,

	["PetDPS: "] = true,

	["DPS"] = true,
	["Pet"] = true,
	["#"] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local tablet = AceLibrary("Tablet-2.0")

local percentString = "(%d%%)"

local playerName = nil
local petName = nil

local event = {
	eventType = "Damage",
	sourceID = "pet",
}

local data = nil
local position = nil
local dirty = 0
local lastUpdateTime
local seconds = nil
local oldDPS = 0

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule("PetDPS", "Parser-3.0")
mod.revision = tonumber(("$Revision: 29069 $"):sub(12, -3)) or 1

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	petName = UnitName("pet")
end

function mod:OnModuleEnable()
	self:RegisterParserEvent(event, "OnParserEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	seconds = 0
	oldDPS = 0
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Pet Damage per Second"] end
function mod:GetBroadcastData() return dirty end

function mod:IsDirty()
	return dirty ~= 0
end

function mod:OnRemoteData(sender, input)
	-- remote dps is placed in the damage table, this is done so we don't need to recalc everybody else's DPS.
	data[sender] = (data[sender] or 0) + input
end

function mod:PLAYER_REGEN_ENABLED()
	lastUpdateTime = nil
end

function mod:OnParserEvent(event)
	assert(event.sourceID == "pet", "Not supposed to get combat events from people except ourselves.")
	if event.amount then
		local newTime = GetTime()
		local timeSinceLastUpdate
		-- on the offhand chance that lastUpdateTime is nil at this point (if somehow we do damage before a regen disabled event fires)
		-- make sure that we have a 
		if not lastUpdateTime then
			timeSinceLastUpdate = 0.1
		else
			timeSinceLastUpdate = newTime - lastUpdateTime
		end
		lastUpdateTime = newTime
		data[playerName] = data[playerName] + event.amount
		seconds = seconds + timeSinceLastUpdate
		local newDPS = data[playerName] / seconds
		dirty = newDPS - oldDPS
		oldDPS = newDPS
	end
end

local function Sorter(a, b) return a[2] > b[2] end
function mod:OnTooltipUpdate()
	if not data then return end

	local tbl = self:new()
	for k, v in pairs(data) do
		if type(v) == "number" and v > 0 then
			if k == playerName then
				table.insert(tbl, self:new(k, v/seconds))
			else
				table.insert(tbl, self:new(k, v))
			end
		end
	end

	local tblCount = #tbl
	if tblCount > 0 then
		local cat = tablet:AddCategory(
			"columns", tblCount > 1 and 4 or 2,
			"text", tblCount > 1 and L["#"] or L["DPS"],
			"text2", tblCount > 1 and L["Pet"] or "",
			"text3", L["DPS"],
			"child_justify", "LEFT",
			"child_justify2", tblCount > 1 and "LEFT" or "RIGHT",
			"child_justify3", "RIGHT",
			"child_justify4", "RIGHT",
			"child_textR", 1,
			"child_textG", 1,
			"child_textB", 0,
			"child_text2R", 1,
			"child_text2G", 1,
			"child_text2B", 1,
			"child_text3R", 1,
			"child_text3G", 1,
			"child_text3B", 1,
			"child_text4R", 0.65,
			"child_text4G", 0.65,
			"child_text4B", 0.65
		)

		local percentage = nil
		if tblCount > 1 then
			table.sort(tbl, Sorter)
			
			local total = 0
			for i, v in ipairs(tbl) do
				total = total + v[2]
			end
			percentage = self:new()
			for i, v in ipairs(tbl) do
				table.insert(percentage, math.floor((v[2] * 100) / total))
			end
		end

		local playerShown = nil
		for i, v in ipairs(tbl) do
			if i > 10 then break end -- Only show the top 10.
			if percentage then
				cat:AddLine(
					"text", tostring(i) .. ".",
					"text2", self:GetColoredName(v[1]),
					"text3", math.floor(v[2]),
					"text4", string.format(percentString, percentage[i])
				)
			else
				cat:AddLine(
					"text", self:GetColoredName(v[1]),
					"text2", math.floor(v[2])
				)
			end
			if v[1] == playerName then
				playerShown = true
				if tblCount > 1 then
					position = i
				else
					position = nil
				end
			end
		end

		percentage = self:del(percentage)

		if not playerShown and data[playerName] > 0 then
			cat:AddLine("text", "?", "text2", self:GetColoredName(playerName), "text3", tostring(data[playerName]/seconds))
			position = "?"
		end
	end

	-- Cleanup.
	tbl = self:del(tbl)
end

function mod:GetDisplayText()
	if data and data[playerName] > 0 then
		return L["PetDPS: "] .. (type(position) ~= "nil" and tostring(position) .. ". " or "") .. math.floor(data[playerName]/seconds)
	end
	return nil
end

